@@ -15,78 +15,85 @@
 #'
 #' @param dataset Data frame. A data frame with different columns is given as
 #' input.
-#' @param nclust Numeric. An integer indicating the number of clusters per
+#' @param min_compression_perc Numeric. An integer indicating the minimum percent compression rate to
+#' be achieved for the dataset
+#' @param n_cells Numeric. An integer indicating the number of cells per
 #' hierarchy (level)
 #' @param depth Numeric. An integer indicating the number of levels. (1 = No
 #' hierarchy, 2 = 2 levels, etc ...)
 #' @param quant.err Numeric. A number indicating the quantization error
-#' threshold.
+#' treshold.
 #' @param projection.scale Numeric. A number indicating the scale factor for
-#' the tessellations so as to visualize the sub-tessellations well enough.
+#' the tesselations so as to visualize the sub-tesselations well enough.
 #' @param normalize Logical. A logical value indicating if the columns in your
 #' dataset should be normalized. Default value is TRUE.
-#' @param distance_metric character. The distance metric can be 'L1_Norm" or "Manhattan". L1_Norm is selected by default.
+#' @param scale_summary List. A list with mean and standard deviation values for all the features in the dataset. 
+#' Pass the scale summary when the input dataset is already scaled or normalize is set to False.
+#' @param distance_metric character. The distance metric can be "L1_Norm" or "Manhattan". L1_Norm is selected by default.
 #' @param error_metric character. The error metric can be "mean" or "max". mean is selected by default
 #' @param quant_method character. The quant_method can be "kmeans" or "kmedoids". kmeans is selected by default
 #' @param diagnose Logical. A logical value indicating if the diagnose is required. Default value is TRUE.
 #' @param hvt_validation Logical. A logical value indicating if the MAD values are to tested for validation set. Default value is FALSE.
 #' @param train_validation_split_ratio Numeric. A numeric value indicating the train and validation split ratio.
-#' @return A list that contains the hierarchical tessellation information. This
+#' @return A list that contains the hierarchical tesselation information. This
 #' list has to be given as input argument to plot the tessellations.
-#' \item{[[1]] }{List. Information about the tessellation co-ordinates - level
+#' \item{[[1]] }{List. Information about the tesselation co-ordinates - level
 #' wise} \item{[[2]] }{List. Information about the polygon co-ordinates - level
 #' wise} 
 #' \item{[[3]] }{List. Information about the hierarchical vector quantized data - level wise}
 #' \item{[[4]] }{List. Information about the model diagnosis- selected level}
 #' \item{[[5]] }{List. Information about the MAD values and percentage anomalies for validation dataset}
 #' 
-#' @author Shubhra Prakash <shubhra.prakash@@mu-sigma.com>, Sangeet Moy Das <sangeet.das@@mu-sigma.com>
+#' @author Shubhra Prakash <shubhra.prakash@@mu-sigma.com>, Sangeet Moy Das <sangeet.das@@mu-sigma.com>, Shantanu Vaidya <shantanu.vaidya@@mu-sigma.com>
 #' @seealso \code{\link{plotHVT}} \cr \code{\link{hvtHmap}}
 #' @keywords hplot
 #' @importFrom magrittr %>%
 #' @examples
 #' data(USArrests)
 #' hvt.results <- list()
-#' hvt.results <- HVT(USArrests, nclust = 15, depth = 1, quant.err = 0.2, 
+#' hvt.results <- HVT(USArrests, min_compression_perc = 70, quant.err = 0.2, 
 #'                    distance_metric = "L1_Norm", error_metric = "mean",
 #'                    projection.scale = 10, normalize = TRUE,
-#'                    quant_method="kmeans",diagnose=TRUE)
+#'                    quant_method="kmeans")
 #' plotHVT(hvt.results, line.width = c(0.8), color.vec = c('#141B41'), 
 #'         maxDepth = 1)
 #'
 #' hvt.results <- list()
-#' hvt.results <- HVT(USArrests, nclust = 15, depth = 3, quant.err = 0.2, 
+#' hvt.results <- HVT(USArrests, n_cells = 15, depth = 3, quant.err = 0.2, 
 #'                    distance_metric = "L1_Norm", error_metric = "mean",
 #'                    projection.scale = 10, normalize = TRUE,
-#'                    quant_method="kmeans",diagnose=TRUE)
+#'                    quant_method="kmeans")
 #' plotHVT(hvt.results, line.width = c(1.2,0.8,0.4), color.vec = c('#141B41','#0582CA','#8BA0B4'), 
 #'         maxDepth = 3)
 #' @export HVT
+
+
 HVT <-
-  function (dataset,
-            nclust = 15,
-            depth = 3,
-            quant.err = 0.2,
-            projection.scale = 10,
-            normalize = TRUE,
-            distance_metric = c("L1_Norm", "L2_Norm"),
-            error_metric = c("mean", "max"),
-            quant_method="kmeans",
-            diagnose=TRUE,
-            hvt_validation=FALSE,
-            train_validation_split_ratio=0.8
+    function(dataset,
+              min_compression_perc = NA,
+              n_cells = NA,
+              depth = 1,
+              quant.err = 0.2,
+              projection.scale = 10,
+              normalize = FALSE,
+              distance_metric = c("L1_Norm", "L2_Norm"),
+              error_metric = c("mean", "max"),
+              quant_method=c("kmeans","kmedoids"),
+              scale_summary = NA,
+              diagnose=FALSE,
+              hvt_validation=FALSE,
+              train_validation_split_ratio=0.8
             ) {
-    # browser()
+    #browser()
     requireNamespace("deldir")       #deldir function
     requireNamespace("Hmisc")        #ceil function
     requireNamespace("grDevices")    #chull function
     requireNamespace("splancs")      #csr function
     requireNamespace("sp")           #point.in.polygon function
-    requireNamespace("conf.design")  #factorize function
-
+    requireNamespace("conf.design")  #factorize functionScreenshot from 2022-10-26 16-15-44
     
-if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatization using K-Medoids is very high for large number of clusters.')}
-# browser()
+if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector quantization using K-Medoids is very high for large number of clusters.')}
+    # browser()
     dataset <- as.data.frame(dataset)
     # dataset <- as.data.frame(sapply(dataset[,1:length(dataset[1,])], as.numeric))
     dataset=data.frame(
@@ -94,39 +101,44 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
       check.names = F,
       row.names = rownames(dataset)
     )
-    
+
+
     ## Diagnose Function
     if(hvt_validation){
       if(train_validation_split_ratio >1 | train_validation_split_ratio <0){stop("The train_validation_split_ratio should be in range 0-1 ")}
       message(paste0("Check MAD parameter has been set to TRUE, the train data will be randomly split by the ratio of  ",train_validation_split_ratio*100,":",(1-train_validation_split_ratio)*100))
-      
+
       set.seed(279)
       smp_size <- floor(train_validation_split_ratio * nrow(dataset))
       train_ind <- sample(seq_len(nrow(dataset)), size = smp_size)
-      
+
       validation_data <- dataset[-train_ind, ]
       dataset <- dataset[train_ind, ]
 
     }
-    
-    
-    
-    
+
+
     if (normalize) {
       scaledata <- scale(dataset, scale = T, center = T)
       rownames(scaledata) <- rownames(dataset)
-      
+
       mean_data <- attr(scaledata, "scaled:center")
       std_data <- attr(scaledata, "scaled:scale")
-      
+
       scale_summary <- list(mean_data = mean_data, std_data = std_data)
-      
+
       # flog.info("scaling is done")
-    } else{
+    } else {
       scaledata <- as.matrix(dataset)
       rownames(scaledata) <- rownames(dataset)
       # flog.info("The data is not scaled as per the user requirement")
-      scale_summary <- NULL
+      if(!all(is.na(scale_summary))){
+        scale_summary = scale_summary
+      }else{
+        scale_summary = NA
+        # stop("Please pass scale_summary to the function or set normalize parameter to TRUE")
+      }
+      
     }
     
     polinfo <- hvqdata <- list()
@@ -134,13 +146,20 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
     hvq_k <-
       hvq(
         scaledata,
-        nclust = nclust,
+        min_compression_perc = min_compression_perc,
+        n_cells = n_cells,
         depth = depth,
         quant.err = quant.err,
         distance_metric = distance_metric,
         error_metric = error_metric,
         quant_method=quant_method
       )
+    
+    if(!is.na(min_compression_perc)){
+      n_cells <- hvq_k$compression_summary$noOfCells
+      depth <- 1
+    }
+    
     # flog.info("HVQ output is ready")
     hvqoutput <- hvq_k$summary
     
@@ -170,20 +189,29 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
     newcols <-  ncol(hvqoutput) - ncol(dataset)
     # Variable to store information on mapping
     par_map <- list()
+   
     for (i in 1:nlevel) {
       #hvqdata segregated according to different levels
-      tessdata[[i]] <- gdata[which(gdata[, "Segment.Level"] == i),]
-      
-      #data to be used as input to sammon function
+      tessdata[[i]] <- gdata[which(gdata[, "Segment.Level"]==i),]
+      # tessdata[[i]] <- tessdata[[i]] %>% mutate(across(where(is.double), round, 8))
+      # tessdata[[i]] <- tessdata[[i]] %>% mutate_if(is.double, round, 8)
+    }
+   
+     # data to be used as input to sammon function
       # input.tessdata[[i]] <- tessdata[[i]][, (newcols+1): ncol(hvqoutput)]
       # d<-cmdscale()
-    }
+   #  }
+   #  }
+   # # tessdata[[i]] <- round(tessdata[[i]], digits = 8)
     counter <- c(1:nlevel)
     
     sammon_par <-
       function(x) {
         10 * (MASS::sammon(
+          # dist() function computes and returns the distance matrix computed by using the specified distance measure
+          #to compute the distances between the rows of a data matrix.
           d = stats::dist(unique(tessdata[[x]][, (newcols + 1):ncol(hvqoutput)])),
+          # d = stats::dist(unique(round(tessdata[[x]], 8)[, (newcols + 1):ncol(hvqoutput)])),
           niter = 10 ^ 5,
           trace = FALSE
         )$points)
@@ -206,19 +234,19 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
       # #sammon datapoints grouped according to the hierarchy
       intermediate.rawdata <- list()
       rn = row.names(points2d[[i]])
-      vec = as.integer(rn) - sum(nclust ^ (0:(i - 1))) + 1
+      vec = as.integer(rn) - sum(n_cells ^ (0:(i - 1))) + 1
       
       # for(j in 1: length(unique(tessdata[[i]][, "Segment.Parent"]))) {
       index = 1
       for (j in unique(tessdata[[i]][, "Segment.Parent"])) {
-        # print(((nclust * (j - 1)) + 1): (j * nclust))
-        # intermediate.rawdata[[j]] <- cbind(points2d[[i]][((nclust * (j - 1)) + 1): (j * nclust), 1],
-        #                                    points2d[[i]][((nclust * (j - 1)) + 1): (j * nclust), 2])
-        if (any(dplyr::between(j - vec / nclust, 0, 0.9999))) {
+        # print(((n_cells * (j - 1)) + 1): (j * n_cells))
+        # intermediate.rawdata[[j]] <- cbind(points2d[[i]][((n_cells * (j - 1)) + 1): (j * n_cells), 1],
+        #                                    points2d[[i]][((n_cells * (j - 1)) + 1): (j * n_cells), 2])
+        if (any(dplyr::between(j - vec / n_cells, 0, 0.9999))) {
           intermediate.rawdata[[index]] <-
-            cbind(points2d[[i]][rn[dplyr::between(j - vec / nclust, 0, 0.9999)], 1],
+            cbind(points2d[[i]][rn[dplyr::between(j - vec / n_cells, 0, 0.9999)], 1],
                   points2d[[i]][rn[dplyr::between(j - vec /
-                                                    nclust, 0, 0.9999)], 2])
+                                                    n_cells, 0, 0.9999)], 2])
           index = index + 1
         }
       }
@@ -246,7 +274,7 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
     #polygon_info is the modified tile.list output except for first level.
     pol_info[[1]] <- deldir::tile.list(deldat1[[1]][[1]])
     
-    #loop throught the polygon and  add the row name as id varible
+    #loop throught the polygon and  add the row name as id variable
     # row names of info in rawdeldata is preserved from the hvqoutput and this row name ties the rawdeldata back to the hierarchy
     # in the hvqoutput dataset
     pol_info[[1]] <- mapply(function(info, id) {
@@ -331,7 +359,7 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
 
       model_train_date=Sys.time()
       input_parameters = list(
-        nclust = nclust,
+        n_cells = n_cells,
         depth = depth,
         quant.err = quant.err,
         projection.scale = projection.scale,
@@ -344,10 +372,6 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
         train_validation_split_ratio = train_validation_split_ratio
       )
       
-      
-      
-      
-      
       validation_result=diag_Suggestion
       fin_out[["model_info"]]=list(model_train_date=model_train_date,
                                    type="hvt_model",
@@ -356,6 +380,10 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
                                    
                                    )
       
+      # generating cell ID using get_cell_id
+      fin_out[[3]]$summary <- get_cell_id(hvt.results=fin_out)
+      fin_out[[3]]$summary <- fin_out[[3]]$summary %>% select(Segment.Level, Segment.Parent, Segment.Child, n, Cell.ID, Quant.Error, colnames(dataset))
+      
       return(fin_out)
       
     } else { 
@@ -371,8 +399,8 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
           # print(tileIndex)
           #a chunk of hvqdata which contains the rows corresponding to a particular parent tile
           gi_par_tiles <-
-            par_tile_indices[[i]][intersect(which((par_tile_indices[[i]] / nclust) <= par_tile_indices[[(i - 1)]][tileIndex]),
-                                            which((par_tile_indices[[(i - 1)]][tileIndex] - 1) < (par_tile_indices[[i]] / nclust)))]
+            par_tile_indices[[i]][intersect(which((par_tile_indices[[i]] / n_cells) <= par_tile_indices[[(i - 1)]][tileIndex]),
+                                            which((par_tile_indices[[(i - 1)]][tileIndex] - 1) < (par_tile_indices[[i]] / n_cells)))]
           gidata <-
             tessdata[[i]][which(tessdata[[i]][, "Segment.Parent"] %in% gi_par_tiles),]
           par_map[[i - 1]][[par_tile_indices[[(i - 1)]][tileIndex]]] <-
@@ -380,12 +408,12 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
           
           #datapoints corresponding to a particular parent tile
           rawdeldati <-
-            rawdeldata[[i]][intersect(which((par_tile_indices[[i]] / nclust) <= par_tile_indices[[(i - 1)]][tileIndex]),
-                                      which((par_tile_indices[[(i - 1)]][tileIndex] - 1) < (par_tile_indices[[i]] / nclust)))]
+            rawdeldata[[i]][intersect(which((par_tile_indices[[i]] / n_cells) <= par_tile_indices[[(i - 1)]][tileIndex]),
+                                      which((par_tile_indices[[(i - 1)]][tileIndex] - 1) < (par_tile_indices[[i]] / n_cells)))]
           if (nrow(gidata) != 0) {
             #transformation of rawdeldata such that they are inside the parent tile and output is stored in new_rawdeldata
             transpoints <-
-              DelaunayInfo(gidata, polygon_info[[(i - 1)]][[tileIndex]], rawdeldati, nclust)
+              DelaunayInfo(gidata, polygon_info[[(i - 1)]][[tileIndex]], rawdeldati, n_cells)
             if (all(transpoints != "-1")) {
               new_rawdeldata[[i]] <- append(new_rawdeldata[[i]], transpoints)
             } else{
@@ -406,15 +434,15 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
         for (tileNo in 1:n_par_tile[[i]]) {
           # print(tileNo)
           #modulus operation for the last index in polygon_info
-          if (((par_tile_indices[[i]][tileNo]) %% nclust)) {
-            last_index <- (par_tile_indices[[i]][tileNo] %% nclust)
+          if (((par_tile_indices[[i]][tileNo]) %% n_cells)) {
+            last_index <- (par_tile_indices[[i]][tileNo] %% n_cells)
           } else{
-            last_index <- nclust
+            last_index <- n_cells
           }
           
           #divide to get the parent tile
           sec_index <-
-            Hmisc::ceil(par_tile_indices[[i]][tileNo] / nclust)
+            Hmisc::ceil(par_tile_indices[[i]][tileNo] / n_cells)
           
           deldat2[[tileNo]] <-
             deldir::deldir(new_rawdeldata[[i]][[tileNo]][, 1],
@@ -509,14 +537,14 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
         
         for (parentIndex in 1:n_par_tile[[i]]) {
           #modulo operation to obtain the last index
-          if (((par_tile_indices[[i]][parentIndex]) %% nclust)) {
-            last_index <- (par_tile_indices[[i]][parentIndex] %% nclust)
+          if (((par_tile_indices[[i]][parentIndex]) %% n_cells)) {
+            last_index <- (par_tile_indices[[i]][parentIndex] %% n_cells)
           } else{
-            last_index <- nclust
+            last_index <- n_cells
           }
           #divide to get second index
           sec_index <-
-            Hmisc::ceil(par_tile_indices[[i]] / nclust)[parentIndex]
+            Hmisc::ceil(par_tile_indices[[i]] / n_cells)[parentIndex]
           
           polygon_info[[i]][[parentIndex]] <-
             Add_boundary_points(polygon_info[[i]][[parentIndex]],
@@ -530,7 +558,7 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
       
       # par_map <- reshape2::melt(par_map)
       # colnames(par_map) <- c('Child','Parent','Level')
-      # par_map <- par_map %>% mutate(ChildNo = (Parent-1)*nclust+Child)
+      # par_map <- par_map %>% mutate(ChildNo = (Parent-1)*n_cells+Child)
       
       fin_out <- list()
       
@@ -614,7 +642,7 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
       
       model_train_date=Sys.time()
       input_parameters = list(
-        nclust = nclust,
+        n_cells = n_cells,
         depth = depth,
         quant.err = quant.err,
         projection.scale = projection.scale,
@@ -634,8 +662,12 @@ if(quant_method=="kmedoids"){message(' K-Medoids: Run time for vector qunatizati
                                    validation_result=validation_result
       )
       
+      # generating cell ID using get_cell_id
+      fin_out[[3]]$summary <- get_cell_id(hvt.results=fin_out)
+      fin_out[[3]]$summary <- fin_out[[3]]$summary %>% select(Segment.Level, Segment.Parent, Segment.Child, n, Cell.ID, Quant.Error, colnames(dataset))
       
       return(fin_out)
+      
     }
     
   }
