@@ -9,4 +9,6 @@ utils::globalVariables(c(".","k","Segment.Level","Segment.Child","Quant.Error",
                          "cellID", "Mean_Absolute_Deviation", "Percentage_of_Anomalous_Points",
                          "hvt.results", "Index", "centroidRadius", "sumOriginal", "Quant.Error.x",
                          "n.x", "n.y", "Quant.Error.y", "Scored.Quant.Error", "anomalyFlag",
-                         "bp.x", "bp.y", "x", "y", "density"))
\ No newline at end of file
+                         "bp.x", "bp.y", "x", "y", "density","Cell.ID","Cell.Number","MapA.Cell.ID","MapA.Segment.Child","MapA.Segment.Level"
+                         ,"MapA.Segment.Parent","MapC.Cell.ID","MapC.Cell.Path","MapC.Segment.Child","MapC.Segment.Level","MapC.Segment.Parent",
+                         "Row.Number","identified_outlier_cells"))
\ No newline at end of file
