@@ -18,7 +18,9 @@
 #' @param x Data Frame. A dataframe of multivariate data. Each row corresponds to an
 #' observation, and each column corresponds to a variable. Missing values are
 #' not accepted.
-#' @param nclust Numeric. Indicating the number of nodes per hierarchy.
+#' @param min_compression_perc Numeric. An integer indicating the minimum percent compression rate to
+#' be achieved for the dataset
+#' @param n_cells Numeric. Indicating the number of nodes per hierarchy.
 #' @param depth Numeric. Indicating the hierarchy depth (or) the depth of the
 #' tree (1 = no hierarchy, 2 = 2 levels, etc..)
 #' @param quant.err Numeric. The quantization error for the algorithm.
@@ -43,23 +45,24 @@
 #' @examples
 #' 
 #' data("USArrests",package="datasets")
-#' hvqOutput = hvq(USArrests, nclust = 5, depth = 2, quant.err = 0.2,
-#' distance_metric='L1_Norm',error_metric='mean')
+#' hvqOutput = hvq(USArrests, n_cells = 5, depth = 2, quant.err = 0.2,
+#' distance_metric='L1_Norm',error_metric='mean',quant_method="kmeans")
 #' 
 #' @export hvq
 #' @keywords internal
+
 hvq <-
   function (x,
-            nclust = 3,
+            min_compression_perc = NA,
+            n_cells = NA,
             depth = 3,
             quant.err = 10,
             algorithm = "Hartigan-Wong",
             distance_metric = c("L1_Norm", "L2_Norm"),
             error_metric = c("mean", "max"),
-            quant_method="kmeans"
+            quant_method=c("kmeans","kmedoids")
   ) {
     
-    
     requireNamespace("dplyr")
     # browser()
     
@@ -73,14 +76,16 @@ hvq <-
     ztab1 <- ztab2 <- ztabn <- NULL
     ztab11 <- ztab12 <- ztab13 <- NULL
     zdepth <- depth
-    quantinit <- rep(F, nclust)
-    ztab3upc<-matrix(0, nrow = ncol(x), nclust)
-    std<-matrix(0, nrow = ncol(x), nclust)
+    if(!is.na(n_cells)){
+      quantinit <- rep(F, n_cells)
+      ztab3upc<-matrix(0, nrow = ncol(x), n_cells)
+      std<-matrix(0, nrow = ncol(x), n_cells)
+    }
     
     set.seed(300)
     # flog.info("Parameters are initialized")
     #outkinit will have centroids and datapoints and size of the cluster
-    # outkinit <- getOptimalCentroids(x, iter.max=100, algorithm=algorithm, nclust,distance_metric=distance_metric,error_metric=error_metric,quant.err=quant.err)
+    # outkinit <- getOptimalCentroids(x, iter.max=100, algorithm=algorithm, n_cells,distance_metric=distance_metric,error_metric=error_metric,quant.err=quant.err)
     
     colSd <- function (x, na.rm=FALSE) apply(X=x, MARGIN=2, FUN=stats::sd, na.rm=na.rm)
     
@@ -99,28 +104,59 @@ hvq <-
     } else if(distance_metric == "L2_Norm"){
       function_to_calculate_distance_metric <- calculate_euclidean_distance_for_each_cluster
     } else{
-      stop('distance_metric must be L1_Norm (Manhattan), L2_Norm(Euclidean) or custom distance function')
+      stop('distance_metric must be L1_Norm (Manhattan), L2_Norm (Euclidean) or a custom distance function')
     }
     
-    ## for error metric i.e; ,mean or max
+    ## for error metric i.e; mean or max
     if(error_metric %in% c("mean","max")){
       function_to_calculate_error_metric <- error_metric
     } else{
       stop('error_metric must be max,mean or custom function')
     }
     
-    
-    outkinit <- getCentroids(x=x, 
-                             kout = stats::kmeans(x, nclust, iter.max=10^5, algorithm=algorithm), 
-                             nclust=nclust,
+    if(!is.na(min_compression_perc)){
+      depth <- 1
+      compress_percent <- 0
+      n_cells_compress_perc = 2
+      num_data_points <- rep(F, n_cells_compress_perc)
+      n_cells_quant_err_list <- list()
+      n_cells = NA
+
+      while(n_cells_compress_perc < nrow(x) & compress_percent <= min_compression_perc){
+      quantinit <- rep(F, n_cells_compress_perc)
+      ztab3upc<-matrix(0, nrow = ncol(x), n_cells_compress_perc)
+      std<-matrix(0, nrow = ncol(x), n_cells_compress_perc)
+      # print(n_cells_compress_perc)
+      outkinit <- getCentroids(x=x,
+                               kout = stats::kmeans(x, n_cells_compress_perc, iter.max=10^5, algorithm=algorithm),
+                               n_cells=n_cells_compress_perc,
+                               function_to_calculate_distance_metric=function_to_calculate_distance_metric,
+                               function_to_calculate_error_metric=function_to_calculate_error_metric,
+                               distance_metric=distance_metric,
+                               quant_method = quant_method)
+
+      n_cells_quant_err_list <- unlist(outkinit$cent)
+      noOfCells = sum(!is.na(n_cells_quant_err_list))
+      noOfCellsBelowQuantizationError = sum(n_cells_quant_err_list < quant.err,na.rm = TRUE)
+      compress_percent = round((noOfCellsBelowQuantizationError/noOfCells) * 100, 2) #percentOfCellsBelowQuantizationErrorThreshold
+      n_cells_compress_perc <- n_cells_compress_perc + 1
+    }
+    n_cells_optimal <- n_cells_compress_perc - 1
+    print(paste0("For the given dataset ",min_compression_perc,"% compression rate is achieved at n_cells : ", n_cells_optimal))
+
+    }else{
+
+      outkinit <- getCentroids(x=x,
+                             kout = stats::kmeans(x, n_cells, iter.max=10^5, algorithm=algorithm),
+                             n_cells=n_cells,
                              function_to_calculate_distance_metric=function_to_calculate_distance_metric,
                              function_to_calculate_error_metric=function_to_calculate_error_metric,
                              distance_metric=distance_metric,
                              quant_method = quant_method)
     
-    # outkinit <- getCentroids(x, kout = stats::kmeans(x, nclust, iter.max=100, algorithm=algorithm), nclust,distance_metric=distance_metric,error_metric=error_metric)
-    # flog.info("Level 1 cluster memberships are calculated")
-    #datapoints grouped into clusters
+      n_cells_optimal <- n_cells
+    }
+
     # names(outkinit$val) <- seq_along(outkinit$val)
     rescl[[1]] <- outkinit$val
     tet <- lapply(outkinit$val, row.names)
@@ -134,11 +170,14 @@ hvq <-
     #     data.frame(tet[[as.character(k)]], 1, 1, k)
     #   else data.frame(`tet..as.character.k...`=character(0), `1`=numeric(0),'1'=numeric(0), k=integer(0))
     # })
+    
+    #length(tet) = no of clusters
     for (k in 1:length(tet)){
       #print(k)
       tet[[k]] <- data.frame(tet[[k]], 1, 1, k)
       #print(head(tet[[k]]))
     }
+    
     names(tet) <- paste(1:length(tet))
     # names(tet) <- seq_along(tet)
     # for (k in 1: length(tet)) 
@@ -156,6 +195,7 @@ hvq <-
     # names(outkinit$values)<-seq_along(outkinit$values)
     initclust <- outkinit$values
     
+    
     if (depth > 1) {
       # flog.info("HVQ calculation started")
       i <- 1
@@ -178,27 +218,35 @@ hvq <-
         #flag which checks quantization error
         quantok <- unlist(resplt[[i]])
         j <- 1
-        while (j < (nclust^i) + 1) {
+        
+        ## Intervention 1
+        while (j < (n_cells^i) + 1) {
           
-          #if datapoint exceeds quantization error and number of datapoints in
-          #that cluster is greater than nclust
-          if (quantok[j] & 
-              NROW(initclust[[j]]) > 3
-          ) {
+          #if datapoint exceeds quantization error and number of datapoints in that cluster is greater than n_cells
+          if (quantok[j] & NROW(initclust[[j]]) > 3) {
             #k-means on the initclust to obtain the next level clustering(sub-clusters)
-            z =data.frame(initclust[[j]])
+            z = data.frame(initclust[[j]])
             # try(
-            outk <- getOptimalCentroids(z, iter.max = 10^5, algorithm = algorithm, nclust,function_to_calculate_distance_metric,function_to_calculate_error_metric,quant.err = quant.err, distance_metric = distance_metric, quant_method=quant_method)
+            # outk <- getOptimalCentroids_new(z, iter.max = 10^5, algorithm = algorithm, n_cells, n_min_points, function_to_calculate_distance_metric, function_to_calculate_error_metric, quant.err = quant.err, distance_metric = distance_metric, quant_method=quant_method)
+            outk <- getOptimalCentroids(z, iter.max = 10^5, algorithm = algorithm, n_cells, function_to_calculate_distance_metric, function_to_calculate_error_metric, quant.err = quant.err, distance_metric = distance_metric, quant_method=quant_method)
             # )
-            # browser()
-            # outk <- getOptimalCentroids(initclust[[j]], iter.max = 100, algorithm = algorithm, nclust,distance_metric = distance_metric,error_metric = error_metric,quant.err = quant.err)
             
-            # outk <- getCentroids(initclust[[j]], kout = stats::kmeans(initclust[[j]], nclust, iter.max = 100, algorithm = algorithm), nclust,distance_metric = distance_metric,error_metric = error_metric)
+            # outk$centers <- outk$centers[-which(sapply(outk$centers, is.na))]
+            # outk$maxQE <- outk$maxQE[-which(sapply(outk$maxQE, is.na))]
+            # outk$meanQE <- outk$meanQE[-which(sapply(outk$meanQE, is.na))]
+            # outk$values <- outk$values[-which(sapply(outk$values, is.na))]
+            # outk$nsize <- outk$nsize[-which(sapply(outk$nsize, is.na))]
+            
+            # browser()
+            # outk <- getOptimalCentroids(initclust[[j]], iter.max = 100, algorithm = algorithm, n_cells,distance_metric = distance_metric,error_metric = error_metric,quant.err = quant.err)
+            
+            # outk <- getCentroids(initclust[[j]], kout = stats::kmeans(initclust[[j]], n_cells, iter.max = 100, algorithm = algorithm), n_cells,distance_metric = distance_metric,error_metric = error_metric)
             #store the datapoints
             # names(outk$val)<-seq_along(outk$val)
             ijrescl[[j]] <- outk$val
             # tet <- lapply(outk$val, sapply, row.names)
             tet <- lapply(outk$val, row.names)
+            tet[-which(sapply(tet, is.null))]
             # names(tet) <- seq_along(tet)
             #create ID's for each datapoint
             # tet <- lapply(1:length(tet),function(k){
@@ -233,7 +281,7 @@ hvq <-
             ijresnsize[[j]] <- outk$nsize
             # ijztab3up[[j]] <- sapply(outk$val, mean)
             
-            ijztab3up[[j]]<-matrix(0, nrow = ncol(x), nclust)
+            ijztab3up[[j]]<-matrix(0, nrow = ncol(x), n_cells)
             
             # rownames(ijztab3up[[j]]) <- colnames(x)
             ijztab3up[[j]] <- t(outk$centroid_val)
@@ -244,20 +292,21 @@ hvq <-
             #store the datapoints
             ijclust <- c(ijclust, outk$values)
           }else {
-            ijrescl[[j]] <- rep(NA, nclust)
+            ijrescl[[j]] <- rep(NA, n_cells)
             ijresid[[j]] <- NULL
-            ijresm[[j]] <- rep(NA, nclust)
-            ijmaxqe[[j]] <- rep(NA, nclust)
-            ijmeanqe[[j]] <- rep(NA, nclust)
-            ijresnsize[[j]] <- rep(0, nclust)
-            ijztab3up[[j]] <- matrix(NA, ncol(x), nclust)
-            ijresplt[[j]] <- rep(F, nclust)
-            ijclust <- c(ijclust, rep(NA, nclust))
+            ijresm[[j]] <- rep(NA, n_cells)
+            ijmaxqe[[j]] <- rep(NA, n_cells)
+            ijmeanqe[[j]] <- rep(NA, n_cells)
+            ijresnsize[[j]] <- rep(0, n_cells)
+            ijztab3up[[j]] <- matrix(NA, ncol(x), n_cells)
+            ijresplt[[j]] <- rep(F, n_cells)
+            ijclust <- c(ijclust, rep(NA, n_cells))
           }
-          ztab1 <- c(ztab1, paste(i + 1, j, 1:nclust, sep = ","))
-          ztab11 <- c(ztab11, rep(i + 1, nclust))
-          ztab12 <- c(ztab12, rep(j, nclust))
-          ztab13 <- c(ztab13, 1: nclust)
+          n_cells_at_j <- length(tet)
+          ztab1 <- c(ztab1, paste(i + 1, j, 1:n_cells_at_j, sep = ","))
+          ztab11 <- c(ztab11, rep(i + 1, n_cells_at_j))
+          ztab12 <- c(ztab12, rep(j, n_cells_at_j))
+          ztab13 <- c(ztab13, 1: n_cells_at_j)
           j <- j + 1
         }
         rescl[[i + 1]] <- ijrescl
@@ -281,22 +330,22 @@ hvq <-
       # flog.info("HVQ for user-defined depth has been calculated")
       #ztab is the output which contains the datapoints and their IDs.
       #initialize ztab
-      ztab <- data.frame(matrix(0, nrow = sum(nclust^(1:zdepth)), 
+      ztab <- data.frame(matrix(0, nrow = sum(n_cells^(1:zdepth)), 
                                 ncol = (ncol(x) + 5)))
       #Segment Level
-      ztab[1:nclust, 1] <- rep(1, nclust)
+      ztab[1:n_cells, 1] <- rep(1, n_cells)
       #Segment Parent
-      ztab[1:nclust, 2] <- rep(1, nclust)
+      ztab[1:n_cells, 2] <- rep(1, n_cells)
       #Segment Child
-      ztab[1:nclust, 3] <- 1: nclust
+      ztab[1:n_cells, 3] <- 1: n_cells
       #Size of the cluster
-      ztab[1:nclust, 4] <- unlist(outkinit$nsize)
+      ztab[1:n_cells, 4] <- unlist(outkinit$nsize)
       #Centroid/Quantization error of the cluster
-      ztab[1:nclust, 5] <- unlist(outkinit$cent)
+      ztab[1:n_cells, 5] <- unlist(outkinit$cent)
       # ztab3upc <- sapply(outkinit$val, mean, na.rm = T)
-      # ztab3upc<-matrix(0, nrow = ncol(x), nclust)
-      # std<-matrix(0, nrow = ncol(x), nclust)
-      for(a in 1: nclust){
+      # ztab3upc<-matrix(0, nrow = ncol(x), n_cells)
+      # std<-matrix(0, nrow = ncol(x), n_cells)
+      for(a in 1: n_cells){
         for(b in 1: ncol(x)){
           ztab3upc[b, a] <- as.matrix(mean(outkinit$val[[a]][, b], na.rm = T))
           rownames(ztab3upc) <- colnames(x)
@@ -310,20 +359,21 @@ hvq <-
         std <- cbind(std, ztab3up[[l]])
       }
       
-      ztab[(nclust + 1):sum(nclust^(1:zdepth)), 1] <- ztab11
-      ztab[(nclust + 1):sum(nclust^(1:zdepth)), 2] <- ztab12
-      ztab[(nclust + 1):sum(nclust^(1:zdepth)), 3] <- ztab13
-      ztab[(nclust + 1):sum(nclust^(1:zdepth)), 4] <- ztab2
+      ztab[(n_cells + 1):sum(n_cells^(1:zdepth)), 1] <- ztab11
+      ztab[(n_cells + 1):sum(n_cells^(1:zdepth)), 2] <- ztab12
+      ztab[(n_cells + 1):sum(n_cells^(1:zdepth)), 3] <- ztab13
+      ztab[(n_cells + 1):sum(n_cells^(1:zdepth)), 4] <- ztab2
       ztab[, 6: ncol(ztab)] <- t(ztab3upc)
-      ztab[(nclust + 1): sum(nclust^(1: zdepth)), 5] <- ztabn
+      ztab[(n_cells + 1): sum(n_cells^(1: zdepth)), 5] <- ztabn
       names(ztab) <- c("Segment.Level", "Segment.Parent", "Segment.Child", 
                        "n", "Quant.Error", colnames(x))
     } else {
-      ztab <- data.frame(matrix(0, nrow = nclust, ncol = (ncol(x) + 
+      # else loop if depth = 1
+      ztab <- data.frame(matrix(0, nrow = n_cells_optimal, ncol = (ncol(x) + 
                                                             5)))
-      ztab[, 1] <- rep(1, nclust)
-      ztab[, 2] <- rep(1, nclust)
-      ztab[, 3] <- 1:nclust
+      ztab[, 1] <- rep(1, n_cells_optimal)
+      ztab[, 2] <- rep(1, n_cells_optimal)
+      ztab[, 3] <- 1:n_cells_optimal
       ztab[, 4] <- unlist(outkinit$nsize)
       ztab[, 5] <- unlist(outkinit$cent)
       ztab_mean <- t(sapply(outkinit$val, colMeans, 
@@ -354,7 +404,7 @@ hvq <-
     ## Calculate compress percentage
     compression_summary <- ztab %>% 
       dplyr::group_by(Segment.Level) %>% 
-      dplyr::summarise(noOfCells = sum(!is.na(Quant.Error)), noOfCellsBelowQuantizationError = sum(Quant.Error < quant.err,na.rm = TRUE)) %>% dplyr::mutate(percentOfCellsBelowQuantizationErrorThreshold = (noOfCellsBelowQuantizationError/noOfCells))
+      dplyr::summarise( noOfCells = sum(!is.na(Quant.Error)), noOfCellsBelowQuantizationError = sum(Quant.Error < quant.err,na.rm = TRUE)) %>% dplyr::mutate(percentOfCellsBelowQuantizationErrorThreshold = (noOfCellsBelowQuantizationError/noOfCells))
     
     colnames(compression_summary)[1] <- "segmentLevel"
     
@@ -363,7 +413,7 @@ hvq <-
     }
  
 
-    compression_summary$parameters=paste("nclust:",nclust,"quant.err:",quant.err,"distance_metric:",distance_metric[1],"error_metric:",error_metric[1],"quant_method:",quant_method[1])
+    compression_summary$parameters=paste("n_cells:",n_cells_optimal,"quant.err:",quant.err,"distance_metric:",distance_metric[1],"error_metric:",error_metric[1],"quant_method:",quant_method[1])
 
     
     ridnames <- resid
@@ -372,4 +422,4 @@ hvq <-
                 error.quant = resm, max_QE = maxqe, plt.clust = resplt, summary = ztab, compression_summary = compression_summary,
                 meanClust = meanCol, sdClust = std,mean_QE = meanqe ))
     
-  }
+  }
\ No newline at end of file
